#ifndef _DGL_H_
#define _DGL_H_

#include "GLES/gl.h"

/* Definitions not in OpenGL ES but in normal OpenGL */

typedef double GLdouble; /* double precision float */
typedef double GLclampd; /* double precision float in [0,1] */

#define GL_INT					0x1404
#define GL_DOUBLE				0x140A

#define GL_POINT_SPRITE_ARB               0x8861
#define GL_COORD_REPLACE_ARB              0x8862

/* Desktop OpenGL functions */

struct DGL
{
	void (*glClearColor)( GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha );

	void (*glClear)( GLbitfield mask );

	void (*glIndexMask)( GLuint mask );

	void (*glColorMask)( GLboolean red, GLboolean green, GLboolean blue, GLboolean alpha );

	void (*glAlphaFunc)( GLenum func, GLclampf ref );

	void (*glBlendFunc)( GLenum sfactor, GLenum dfactor );

	void (*glLogicOp)( GLenum opcode );

	void (*glCullFace)( GLenum mode );

	void (*glFrontFace)( GLenum mode );

	void (*glPointSize)( GLfloat size );

	void (*glLineWidth)( GLfloat width );

	void (*glPolygonOffset)( GLfloat factor, GLfloat units );

	void (*glScissor)( GLint x, GLint y, GLsizei width, GLsizei height);

	void (*glClipPlane)( GLenum plane, const GLdouble *equation );

	void (*glGetClipPlane)( GLenum plane, GLdouble *equation );

	void (*glEnable)( GLenum cap );

	void (*glDisable)( GLenum cap );

	GLboolean (*glIsEnabled)( GLenum cap );

	void (*glEnableClientState)( GLenum cap );

	void (*glDisableClientState)( GLenum cap );

	void (*glGetBooleanv)( GLenum pname, GLboolean *params );

	void (*glGetDoublev)( GLenum pname, GLdouble *params );

	void (*glGetFloatv)( GLenum pname, GLfloat *params );

	void (*glGetIntegerv)( GLenum pname, GLint *params );

	GLenum (*glGetError)( void );

	const GLubyte* (*glGetString)( GLenum name );

	void (*glFinish)( void );

	void (*glFlush)( void );

	void (*glHint)( GLenum target, GLenum mode );

	void (*glClearDepth)( GLclampd depth );

	void (*glDepthFunc)( GLenum func );

	void (*glDepthMask)( GLboolean flag );

	void (*glDepthRange)( GLclampd near_val, GLclampd far_val );

	void (*glMatrixMode)( GLenum mode );

	void (*glOrtho)( GLdouble left, GLdouble right,
			GLdouble bottom, GLdouble top,
			GLdouble near_val, GLdouble far_val );

	void (*glFrustum)( GLdouble left, GLdouble right,
			GLdouble bottom, GLdouble top,
			GLdouble near_val, GLdouble far_val );

	void (*glViewport)( GLint x, GLint y,
			GLsizei width, GLsizei height );

	void (*glPushMatrix)( void );

	void (*glPopMatrix)( void );

	void (*glLoadIdentity)( void );

	void (*glLoadMatrixd)( const GLdouble *m );
	void (*glLoadMatrixf)( const GLfloat *m );

	void (*glMultMatrixd)( const GLdouble *m );
	void (*glMultMatrixf)( const GLfloat *m );

	void (*glRotated)( GLdouble angle,
			GLdouble x, GLdouble y, GLdouble z );
	void (*glRotatef)( GLfloat angle,
			GLfloat x, GLfloat y, GLfloat z );

	void (*glScaled)( GLdouble x, GLdouble y, GLdouble z );
	void (*glScalef)( GLfloat x, GLfloat y, GLfloat z );

	void (*glTranslated)( GLdouble x, GLdouble y, GLdouble z );

	void (*glTranslatef)( GLfloat x, GLfloat y, GLfloat z );

	void (*glBegin)( GLenum mode );

	void (*glEnd)( void );

#if 0
	void glVertex2d( GLdouble x, GLdouble y );
	void glVertex2f( GLfloat x, GLfloat y );
	void glVertex2i( GLint x, GLint y );
#endif
	void (*glVertex2s)( GLshort x, GLshort y );

#if 0
	void glVertex3d( GLdouble x, GLdouble y, GLdouble z );
	void glVertex3f( GLfloat x, GLfloat y, GLfloat z );
	void glVertex3i( GLint x, GLint y, GLint z );
#endif
	void (*glVertex3s)( GLshort x, GLshort y, GLshort z );

#if 0
	void glVertex4d( GLdouble x, GLdouble y, GLdouble z, GLdouble w );
	void glVertex4f( GLfloat x, GLfloat y, GLfloat z, GLfloat w );
	void glVertex4i( GLint x, GLint y, GLint z, GLint w );
#endif
	void (*glVertex4s)( GLshort x, GLshort y, GLshort z, GLshort w );

#if 0
	void glVertex2dv( const GLdouble *v );
	void glVertex2fv( const GLfloat *v );
	void glVertex2iv( const GLint *v );
	void glVertex2sv( const GLshort *v );

	void glVertex3dv( const GLdouble *v );
	void glVertex3fv( const GLfloat *v );
	void glVertex3iv( const GLint *v );
	void glVertex3sv( const GLshort *v );

	void glVertex4fv( const GLfloat *v );
	void glVertex4iv( const GLint *v );
	void glVertex4sv( const GLshort *v );


	void glNormal3b( GLbyte nx, GLbyte ny, GLbyte nz );
#endif
	void (*glVertex4dv)( const GLdouble *v );

	void (*glNormal3d)( GLdouble nx, GLdouble ny, GLdouble nz );
	void (*glNormal3f)( GLfloat nx, GLfloat ny, GLfloat nz );
	void (*glNormal3fv)( const GLfloat *v );

#if 0
	void glNormal3i( GLint nx, GLint ny, GLint nz );
	void glNormal3s( GLshort nx, GLshort ny, GLshort nz );

	void glNormal3bv( const GLbyte *v );
	void glNormal3dv( const GLdouble *v );
	void glNormal3iv( const GLint *v );
	void glNormal3sv( const GLshort *v );

	void glColor3b( GLbyte red, GLbyte green, GLbyte blue );
	void glColor3d( GLdouble red, GLdouble green, GLdouble blue );
	void glColor3f( GLfloat red, GLfloat green, GLfloat blue );
	void glColor3i( GLint red, GLint green, GLint blue );
	void glColor3s( GLshort red, GLshort green, GLshort blue );
	void glColor3ub( GLubyte red, GLubyte green, GLubyte blue );
	void glColor3ui( GLuint red, GLuint green, GLuint blue );
	void glColor3us( GLushort red, GLushort green, GLushort blue );

	void glColor4b( GLbyte red, GLbyte green,
			GLbyte blue, GLbyte alpha );
#endif
	void (*glColor4fv)( const GLfloat *red );

	void (*glColor4d)( GLdouble red, GLdouble green,
			GLdouble blue, GLdouble alpha );

	void (*glColor4f)( GLfloat red, GLfloat green,
			GLfloat blue, GLfloat alpha );

#if 0
	void glColor4i( GLint red, GLint green,
			GLint blue, GLint alpha );
	void glColor4s( GLshort red, GLshort green,
			GLshort blue, GLshort alpha );
#endif

	void (*glColor4ub)( GLubyte red, GLubyte green,
			GLubyte blue, GLubyte alpha );

#if 0
	void glColor4ui( GLuint red, GLuint green,
			GLuint blue, GLuint alpha );
	void glColor4us( GLushort red, GLushort green,
			GLushort blue, GLushort alpha );


	void glColor3bv( const GLbyte *v );
	void glColor3dv( const GLdouble *v );
	void glColor3fv( const GLfloat *v );
	void glColor3iv( const GLint *v );
	void glColor3sv( const GLshort *v );
	void glColor3ubv( const GLubyte *v );
	void glColor3uiv( const GLuint *v );
	void glColor3usv( const GLushort *v );

	void glColor4bv( const GLbyte *v );
	void glColor4dv( const GLdouble *v );
	void glColor4fv( const GLfloat *v );
	void glColor4iv( const GLint *v );
	void glColor4sv( const GLshort *v );
	void glColor4ubv( const GLubyte *v );
	void glColor4uiv( const GLuint *v );
	void glColor4usv( const GLushort *v );


	void glTexCoord1d( GLdouble s );
	void glTexCoord1f( GLfloat s );
	void glTexCoord1i( GLint s );
	void glTexCoord1s( GLshort s );

	void glTexCoord2d( GLdouble s, GLdouble t );
	void glTexCoord2f( GLfloat s, GLfloat t );
	void glTexCoord2i( GLint s, GLint t );
	void glTexCoord2s( GLshort s, GLshort t );

	void glTexCoord3d( GLdouble s, GLdouble t, GLdouble r );
	void glTexCoord3f( GLfloat s, GLfloat t, GLfloat r );
	void glTexCoord3i( GLint s, GLint t, GLint r );
	void glTexCoord3s( GLshort s, GLshort t, GLshort r );

	void glTexCoord4d( GLdouble s, GLdouble t, GLdouble r, GLdouble q );
	void glTexCoord4f( GLfloat s, GLfloat t, GLfloat r, GLfloat q );
	void glTexCoord4i( GLint s, GLint t, GLint r, GLint q );
	void glTexCoord4s( GLshort s, GLshort t, GLshort r, GLshort q );

	void glTexCoord1dv( const GLdouble *v );
	void glTexCoord1fv( const GLfloat *v );
	void glTexCoord1iv( const GLint *v );
	void glTexCoord1sv( const GLshort *v );

	void glTexCoord2dv( const GLdouble *v );
	void glTexCoord2fv( const GLfloat *v );
	void glTexCoord2iv( const GLint *v );
	void glTexCoord2sv( const GLshort *v );

	void glTexCoord3dv( const GLdouble *v );
	void glTexCoord3fv( const GLfloat *v );
	void glTexCoord3iv( const GLint *v );
	void glTexCoord3sv( const GLshort *v );

	void glTexCoord4dv( const GLdouble *v );
	void glTexCoord4fv( const GLfloat *v );
	void glTexCoord4iv( const GLint *v );
	void glTexCoord4sv( const GLshort *v );


	void glRasterPos2d( GLdouble x, GLdouble y );
	void glRasterPos2f( GLfloat x, GLfloat y );
	void glRasterPos2i( GLint x, GLint y );
	void glRasterPos2s( GLshort x, GLshort y );

	void glRasterPos3d( GLdouble x, GLdouble y, GLdouble z );
	void glRasterPos3f( GLfloat x, GLfloat y, GLfloat z );
	void glRasterPos3i( GLint x, GLint y, GLint z );
	void glRasterPos3s( GLshort x, GLshort y, GLshort z );

	void glRasterPos4d( GLdouble x, GLdouble y, GLdouble z, GLdouble w );
	void glRasterPos4f( GLfloat x, GLfloat y, GLfloat z, GLfloat w );
	void glRasterPos4i( GLint x, GLint y, GLint z, GLint w );
	void glRasterPos4s( GLshort x, GLshort y, GLshort z, GLshort w );

	void glRasterPos2dv( const GLdouble *v );
	void glRasterPos2fv( const GLfloat *v );
	void glRasterPos2iv( const GLint *v );
	void glRasterPos2sv( const GLshort *v );

	void glRasterPos3dv( const GLdouble *v );
	void glRasterPos3fv( const GLfloat *v );
	void glRasterPos3iv( const GLint *v );
	void glRasterPos3sv( const GLshort *v );

	void glRasterPos4dv( const GLdouble *v );
	void glRasterPos4fv( const GLfloat *v );
	void glRasterPos4iv( const GLint *v );
	void glRasterPos4sv( const GLshort *v );
#endif

	void (*glVertexPointer)( GLint size, GLenum type,
			GLsizei stride, const GLvoid *ptr );

	void (*glNormalPointer)( GLenum type, GLsizei stride,
			const GLvoid *ptr );

	void (*glColorPointer)( GLint size, GLenum type,
			GLsizei stride, const GLvoid *ptr );

#if 0
	void glIndexPointer( GLenum type, GLsizei stride,
			const GLvoid *ptr );
#endif

	void (*glTexCoordPointer)( GLint size, GLenum type,
			GLsizei stride, const GLvoid *ptr );

	void (*glDrawElements)( GLenum mode, GLsizei count,
			GLenum type, const GLvoid *indices );

	void (*glShadeModel)( GLenum mode );

	void (*glLightf)( GLenum light, GLenum pname, GLfloat param );
	void (*glLightfv)( GLenum light, GLenum pname, const GLfloat *params );

	void (*glGetLightfv)( GLenum light, GLenum pname,
			GLfloat *params );

	void (*glLightModelf)( GLenum pname, GLfloat param );
	void (*glLightModelfv)( GLenum pname, const GLfloat *params );

	void (*glMaterialf)( GLenum face, GLenum pname, GLfloat param );
	void (*glMaterialfv)( GLenum face, GLenum pname, const GLfloat *params );

	void (*glGetMaterialfv)( GLenum face, GLenum pname, GLfloat *params );

	void (*glPixelStorei)( GLenum pname, GLint param );

	void (*glReadPixels)( GLint x, GLint y,
			GLsizei width, GLsizei height,
			GLenum format, GLenum type,
			GLvoid *pixels );

#if 0
	void glDrawPixels( GLsizei width, GLsizei height,
			GLenum format, GLenum type,
			const GLvoid *pixels );
#endif

	void (*glStencilFunc)( GLenum func, GLint ref, GLuint mask );

	void (*glStencilMask)( GLuint mask );

	void (*glStencilOp)( GLenum fail, GLenum zfail, GLenum zpass );

	void (*glClearStencil)( GLint s );

	void (*glTexEnvf)( GLenum target, GLenum pname, GLfloat param );
	void (*glTexEnvi)( GLenum target, GLenum pname, GLint param );

	void (*glTexEnvfv)( GLenum target, GLenum pname, const GLfloat *params );
	void (*glTexEnviv)( GLenum target, GLenum pname, const GLint *params );

	void (*glGetTexEnvfv)( GLenum target, GLenum pname, GLfloat *params );
	void (*glGetTexEnviv)( GLenum target, GLenum pname, GLint *params );

	void (*glTexParameterf)( GLenum target, GLenum pname, GLfloat param );
	void (*glTexParameteri)( GLenum target, GLenum pname, GLint param );

	void (*glGetTexParameterfv)(GLenum target, GLenum pname, GLfloat *params);
	void (*glGetTexParameteriv)(GLenum target, GLenum pname, GLint *params);

#if 0
	void glGetTexLevelParameterfv( GLenum target, GLint level,
			GLenum pname, GLfloat *params );
	void glGetTexLevelParameteriv( GLenum target, GLint level,
			GLenum pname, GLint *params );
#endif

	void (*glTexImage2D)( GLenum target, GLint level,
			GLint internalFormat,
			GLsizei width, GLsizei height,
			GLint border, GLenum format, GLenum type,
			const GLvoid *pixels );

	void (*glGenTextures)( GLsizei n, GLuint *textures );

	void (*glDeleteTextures)( GLsizei n, const GLuint *textures);

	void (*glBindTexture)( GLenum target, GLuint texture );

	GLboolean (*glIsTexture)( GLuint texture );

	void (*glTexSubImage2D)( GLenum target, GLint level,
			GLint xoffset, GLint yoffset,
			GLsizei width, GLsizei height,
			GLenum format, GLenum type,
			const GLvoid *pixels );

	void (*glCopyTexImage2D)( GLenum target, GLint level,
			GLenum internalformat,
			GLint x, GLint y,
			GLsizei width, GLsizei height,
			GLint border );

	void (*glCopyTexSubImage2D)( GLenum target, GLint level,
			GLint xoffset, GLint yoffset,
			GLint x, GLint y,
			GLsizei width, GLsizei height );

	void (*glFogi)( GLenum pname, GLint param );
	void (*glFogf)( GLenum pname, GLfloat param );
	void (*glFogfv)( GLenum pname, const GLfloat *params );

	void (*glActiveTexture)( GLenum texture );

	void (*glClientActiveTexture)( GLenum texture );

	void (*glMultiTexCoord4f)( GLenum target, GLfloat s, GLfloat t, GLfloat r, GLfloat q );
	void (*glMultiTexCoord4d)( GLenum target, GLdouble s, GLdouble t, GLdouble r, GLdouble q );

	void (*glSampleCoverage)( GLclampf value, GLboolean invert );

	void (*glBindBuffer)(GLenum, GLuint);
	void (*glDeleteBuffers)(GLsizei, const GLuint *);
	void (*glGenBuffers)(GLsizei, GLuint *);
	GLboolean (*glIsBuffer)(GLuint);
	void (*glBufferData)(GLenum, GLsizeiptr, const GLvoid *, GLenum);
	void (*glBufferSubData)(GLenum, GLintptr, GLsizeiptr, const GLvoid *);
	void (*glGetBufferSubData)(GLenum, GLintptr, GLsizeiptr, GLvoid *);
	void (*glGetBufferParameteriv)(GLenum target, GLenum pname, GLint *params);

	void (*glPointParameterf)(GLenum, GLfloat);
	void (*glPointParameterfv)(GLenum, const GLfloat *);
};

void *_dgles_platform_open();
void _dgles_platform_close(void *);
void *_dgles_platform_sym(void *, const char *);

#endif
